#!/usr/bin/env ruby

# The media_wiki gem is called mediawiki-gateway and needs to be installed by hand
# it also requires Ruby < 2 I think.

require "digest"
require "mysql2"
require 'media_wiki'

TITLE_FIELD = "#[Title]#"
DB_NAME = "dradispro"
DB_USER = "dradispro_db"
DB_PASSWORD = "dradispro_db_pass"
DB_PORT = 3307

MEDIAWIKI_URL = 'http://192.168.0.1/api.php'
MEDIAWIKI_USER = "robin"
MEDIAWIKI_PASSWORD = "x"

seen_hashes = []
total_added = 0

# If you want to draw together multiple Dradis VMs then set up
# ssh tunnels to each of them on consecutive ports and enable
# the following for loop. Don't forget the end at the end of 
# the script.
#
# You then need to change the :port => DB_PORT entry in the connection
# string to be :port => port

#3307.upto(3307) do |port|
	#puts "Port " + port.to_s
	begin
		client = Mysql2::Client.new(:host => "127.0.0.1", :username => DB_USER, :password => DB_PASSWORD, :port => DB_PORT, :database => DB_NAME)
	rescue Mysql2::Error
		puts "Could not connect to the database, have you forwarded the connection?"
		puts "Use:"
		puts "ssh dradispro@dradispro.local -L 3307:localhost:3306"
		puts
		exit
	end

	mw = MediaWiki::Gateway.new(MEDIAWIKI_URL)
	begin
		mw.login(MEDIAWIKI_USER, MEDIAWIKI_PASSWORD)
	rescue MediaWiki::Unauthorized
		puts "MediaWiki login failed"
		puts
		exit
	end

	results = client.query("select id from categories where name = 'Issue description';")
	issue_category_id = results.first["id"]
	puts "Issue category ID = " + issue_category_id.to_s

	results = client.query("select text from notes where category_id = " + issue_category_id.to_s + ";")
	results.each do |row|
		issue = row["text"]
		title_next = false
		hash = Digest::SHA1.hexdigest(issue)
		if seen_hashes.include? hash
			#puts "Already Seen"
		else
	#		puts issue
			title = nil
			issue.each_line do |line|
				if title_next
					title = line.strip
					puts "Adding: " + title
					title_next = false
				end
				if line.strip == TITLE_FIELD
					title_next = true
				end
			end
			if title.nil?
				title = issue.lines.first.strip
				puts "No title found, using first line: " + title
			end
			if title == ""
				title = "Untitled " + Random.rand(1024)
			end
			seen_hashes << hash
			# Can't start with a hash, probably other special characters
			# as well so just add them here
			if title[0] == "#"
				title = title[1..title.length]
			end
			# max length of the title is 255 characters
			title = title[0..254]
	#		puts "issue: " + issue
			added_or_fail = false
			until added_or_fail
				begin
					puts "Title: " + title
					mw.create(title, issue, :summary => title)
					puts "Added successfully"
					added_or_fail = true
					total_added += 1
				rescue MediaWiki::APIError => e
					# Already created so change the title and add again so both go in
					if e.to_s =~ /The article you tried to create has been created already/
						puts "Entry already exists so trying again with new title"
						# have to add the number to the start as mediawiki isn't happy if
						# a title already ends in a number and you then add another number
						title = Random.rand(1024).to_s + " " + title[0..250]
					else
						added_or_fail = true
						puts "There was an error:"
						puts e.to_s
					end

				rescue MediaWiki::Unauthorized
					puts "Something failed and they didn't tell me why"
					added_or_fail = true
				end
			end
		end
	end
# For the multiple port loop
#end

puts 
puts "Finished adding. " + total_added.to_s + " entries created"
puts
